@@ -162,3 +162,50 @@ lev2var <- function(x, ...){
     z
   }
 }
+
+
+has.intercept <- function(object, ...) {
+  UseMethod("has.intercept")
+}
+
+has.intercept.default <- function(object, ...) {
+  has.intercept(formula(object), ...)
+}
+
+has.intercept.formula <- function(object, ...) {
+  attr(terms(object), "intercept") == 1L
+}
+
+has.intercept.Formula <- function(object, rhs = NULL, ...) {
+  ## NOTE: return a logical vector of the necessary length
+  ## (which might be > 1)
+  if(is.null(rhs)) rhs <- 1:length(attr(object, "rhs"))
+  sapply(rhs, function(x) has.intercept(formula(object, lhs = 0, rhs = x)))
+}
+
+pres <- function(x) {  # pres.panelmodel
+  ## extracts model residuals as pseries
+
+    ## extract indices
+    groupind<-attr(x$model, "index")[,1]  #attr(model.frame(x), "index")[[1]]
+    timeind<-attr(x$model, "index")[,2]   #attr(model.frame(x), "index")[[2]]
+    if(!is.null(x$args$model)) {  # fix to allow operation with pggls, pmg
+    ## Achim's fix
+     if(x$args$model == "fd") {
+       groupi <- as.numeric(groupind)
+       ## make vector =1 on first obs in each group, 0 elsewhere
+       selector <-groupi-c(0,groupi[-length(groupi)])
+       selector[1] <- 1 # the first must always be 1
+       ## eliminate first obs in time for each group
+       groupind <- groupind[!selector]
+       timeind <- timeind[!selector]
+     }
+ }
+
+  resdata <- data.frame(ee=x$residuals, ind=groupind, tind=timeind)
+  pee<-pdata.frame(resdata, index=c("ind","tind"))
+
+  pres <- pee$ee
+  return(pres)
+}
+
