@@ -1,8 +1,31 @@
+starX <- function(formula, data, model, rhs = 1, effect){
+    apdim <- pdim(data)
+    amatrix <- model.matrix(formula, data, model, effect, rhs)
+    T <- length(unique(index(data, 2)))
+    N <- length(unique(index(data, 1)))
+    if (apdim$balanced){
+        result <- Reduce("cbind",
+                        lapply(seq_len(ncol(amatrix)),
+                               function(x)
+                               matrix(amatrix[, x], ncol = T, byrow = TRUE)[rep(1:N, each = T), ]))
+    }
+    else{
+        Ti <- apdim$Tint$Ti
+        result <- lapply(seq_len(ncol(amatrix)), function(x)
+                     structure(amatrix[, x], index = index(data), class = c("pseries", class(amatrix[, x]))))
+        result <- Reduce("cbind", lapply(result, as.matrix))
+        result <- result[rep(1:N, times = Ti), ]
+        result[is.na(result)] <- 0
+    }
+    result
+}   
+
 plm <-  function(formula, data, subset, na.action,
                  effect=c('individual','time','twoways'),
                  model = c('within','random','ht','between','pooling','fd'),
                  random.method = c('swar','walhus','amemiya','nerlove', 'kinla'),
-                 inst.method = c('bvk','baltagi'),
+                 random.dfcor = NULL,
+                 inst.method = c('bvk','baltagi', 'am', 'bmc'),
                  restrict.matrix = NULL,
                  restrict.rhs = NULL,
                  index = NULL,
@@ -27,6 +50,7 @@ plm <-  function(formula, data, subset, na.action,
   random.method <- match.arg(random.method)
   inst.method <- match.arg(inst.method)
 
+  # deprecated, pht is no longer maintained
   if (!is.na(model) && model == "ht"){
     ht <- match.call(expand.dots=FALSE)
     m <- match(c("formula", "data", "subset", "na.action", "index"),names(ht),0)
@@ -45,13 +69,13 @@ plm <-  function(formula, data, subset, na.action,
   }
 
   # Check whether data is a pdata.frame and if not create it
+  orig_rownames <- row.names(data)
   if (!inherits(data, "pdata.frame")) data <- pdata.frame(data, index)
   # Create a Formula object if necessary
   if (!inherits(formula, "pFormula")) formula <- pFormula(formula)
 
   # in case of 2part formula, check whether the second part should be
   # updated, e.g. y~x1+x2+x3|.-x2+z becomes y~x1+x2+x3|x1+x3+z
-
   # length(formula)[2] because the length is now a vector of length 2
   if (length(formula)[2] == 2) formula <- expand.formula(formula)
 
@@ -67,25 +91,32 @@ plm <-  function(formula, data, subset, na.action,
   # eval in parent.frame() doesn't work
   #  data <- eval(mf, sys.frame(which = nframe))
   data <- eval(mf, parent.frame())
+  
+  # preserve original row.names for data [also fancy rownames]; so functions like
+  # pmodel.response(), model.frame(), model.matrix(), residuals() return the original row.names
+  # eval(mf, parent.frame()) returns row.names as character vector containing
+  # "row_number" with incomplete observations dropped
+  row.names(data) <- orig_rownames[as.numeric(row.names(data))]
   # return the model.frame or estimate the model
   if (is.na(model)){
     attr(data, "formula") <- formula
     return(data)
   }
   args <- list(model = model, effect = effect,
-               random.method = random.method, inst.method = inst.method)
-  result <- plm.fit(formula, data, model, effect, random.method, inst.method)
+               random.method = random.method, random.dfcor = random.dfcor,
+               inst.method = inst.method)
+  result <- plm.fit(formula, data, model, effect, random.method, random.dfcor, inst.method)
   result$call <- cl
   result$args <- args
   result
 }
 
-plm.fit <- function(formula, data, model, effect, random.method, inst.method){
+plm.fit <- function(formula, data, model, effect, random.method, random.dfcor, inst.method){
   # if a random effect model is estimated, compute the error components
   if (model == "random"){
     pdim <- pdim(data)
     is.balanced <- pdim$balanced
-    estec <- ercomp(formula, data, effect, method = random.method)
+    estec <- ercomp(formula, data, effect, method = random.method, dfcor = random.dfcor)
     sigma2 <- estec$sigma2
     theta <- estec$theta
     index <- attr(data, "index")
@@ -97,37 +128,31 @@ plm.fit <- function(formula, data, model, effect, random.method, inst.method){
   }
   # extract the model.matrix and the model.response
   X <- model.matrix(formula, data, rhs = 1, model = model, effect = effect, theta = theta)
-   if (ncol(X) == 0)
-      stop("empty model")
-    y <- pmodel.response(formula, data, model = model, effect = effect, theta = theta)
+  if (ncol(X) == 0) stop("empty model")
+  y <- pmodel.response(formula, data, model = model, effect = effect, theta = theta)
   ## extract the matrix of instruments if necessary
   if (length(formula)[2] > 1){
-    if (length(formula)[2] == 2)
-      W <- model.matrix(formula, data, rhs = 2, model = model, effect = effect, theta = theta)
-    else
-      W <- model.matrix(formula, data, rhs = c(2, 3), model = model, effect = effect, theta = theta)
-    if (model == "random"){
-      X <- X / sqrt(sigma2$idios)
-      y <- y / sqrt(sigma2$idios)
-      Wb <- model.matrix(formula, data, rhs = 2, model = "Between", effect = effect)
-      Ww <- model.matrix(formula, data, rhs = 2, model = "within", effect = effect)
-      if (length(formula)[2] == 3){
-        WOw <- model.matrix(formula, data, rhs = 3, model = "within", effect = effect)
-        Ww <- cbind(Ww, WOw)
+      if (length(formula)[2] == 2) W <- model.matrix(formula, data, rhs = 2, model = model, effect = effect, theta = theta)
+      else W <- model.matrix(formula, data, rhs = c(2, 3), model = model, effect = effect, theta = theta)
+      if (model == "random" && inst.method != "bvk"){
+          X <- X / sqrt(sigma2$idios)
+          y <- y / sqrt(sigma2$idios)
+          W1 <- model.matrix(formula, data, rhs = 2, model = "within", effect = effect, theta = theta)
+          B1 <- model.matrix(formula, data, rhs = 2, model = "Between", effect = effect, theta = theta)
+          StarW1 <- starX(formula, data, rhs = 2, model = "within", effect = effect)
+          if (length(formula)[2] == 3){
+              W2 <- model.matrix(formula, data, rhs = 3, model = "within", effect = effect, theta = theta)
+              StarW2 <- starX(formula, data, rhs = 3, model = "within", effect = effect)
+          }
+          else W2 <- StarW2 <- NULL
+          if (inst.method == "baltagi") W <- cbind(W1, W2, B1)
+          if (inst.method == "am") W <- cbind(W1, W2, B1, StarW1)
+          if (inst.method == "bmc") W <- cbind(W1, W2, B1, StarW1, StarW2)
+          # quick and dirty trick to remove columns of 0
+          zerovars <- apply(W, 2, function(x) max(abs(x), na.rm = TRUE)) < 1E-5
+          W <- W[, !zerovars]
       }
-      if(inst.method == "baltagi") W <- cbind(Ww, Wb)
-      if (! is.balanced) sig2one <- sigma2$one[as.character(cond)] else sig2one <- sigma2$one
-      if(inst.method == "bvk"){
-        # to construct the W matrix, we must take into account the fact that 
-        # ncol(Wb) >= ncol(Ws)
-        W <- Wb / sqrt(sig2one)
-        W[, colnames(Ww)] <- W[, colnames(Ww)]+ Ww / sqrt(sigma2$idios)
-      }
-      # quick and dirty trick to remove columns of 0
-      zerovars <- apply(W, 2, function(x) max(abs(x), na.rm = TRUE)) < 1E-5
-      W <- W[, !zerovars]
-    }
-    if (ncol(W) < ncol(X)) stop("insufficient number of instruments")
+      if (ncol(W) < ncol(X)) stop("insufficient number of instruments")
   }
   else W <- NULL
   # compute the estimation
@@ -179,84 +204,3 @@ mylm <- function(y, X, W = NULL){
   result
 }
 
-## plm.models <- function(sysplm, amodel, ...){
-##     # run plm for each equations of the list, store the results in a
-##     # list
-##     formulas <- sysplm$formula
-##     L <- length(formulas) - 1
-##     models <- .residuals <- vector(mode = "list", length = L)
-##     for (l in 2:(L+1)){
-##       aformula <- as.formula(formulas[[l]])
-##       sysplm$formula <- aformula
-##       sysplm[[1]] <- as.name("plm")
-##       sysplm$model <- amodel
-##       models[[l-1]] <- eval(sysplm, parent.frame())
-##     }
-##     models
-##   }
-##   modelWithin <- plm.models(sysplm, amodel = "within")
-##   modelBetween <- plm.models(sysplm, "between")
-##   modelRandom <- plm.models(sysplm, amodel = "random", random.method = "kinla")
-
-##   # Get the residuals and compute the consistent estimation of the
-##   # covariance matrix of the residuals
-##   ResWithin <- sapply(modelWithin, resid)
-##   ResBetween <- sapply(modelBetween, resid)
-
-##   SW <-crossprod(ResWithin) / nrow(ResWithin)
-##   SB <- crossprod(ResBetween) / nrow(ResBetween)
-  
-##   SWm05 <- chol(SW)
-##   SBm05 <- chol(SB)
-  
-##   # Extract the model matrix and the response and transform them in
-##   # order to get clean errors
-##   XW <- lapply(modelWithin, model.matrix)
-##   WW <- lapply(modelWithin, function(x) model.matrix(formula(x), model.frame(x), rhs = )
-##   yW <- lapply(modelWithin, function(x) pmodel.response(x))
-##   XB <- lapply(modelWithin, function(x) model.matrix(x, model="Between"))
-##   yB <- lapply(modelBetween, function(x) pmodel.response(x, model="Between"))
-
-  
-##   BIG <- function(X, y, S){
-##     N <- length(y[[1]])
-##     BIGX <- c()
-##     BIGy <- c()
-##     L <- nrow(S)
-##     for (l in 1:L){
-##       rowBIGy <- rep(0, N)
-##       rowBIGX <- c()
-##       for (m in 1:L){
-##         rowBIGX <- cbind(rowBIGX, S[l, m] * X[[m]])
-##         rowBIGy <- rowBIGy + S[l, m] * y[[m]]
-##       }
-##       BIGX <- rbind(BIGX, rowBIGX)
-##       BIGy <- c(BIGy, rowBIGy)
-##     }
-##     return(list(X = BIGX, y = BIGy))
-##   }
-
-##   XW <- lapply(XW, function(x) cbind(0, x))
-  
-##   BIGW <- BIG(XW, yW, SWm05)
-##   BIGB <- BIG(XB, yB, SBm05)
-##   # Compute the ols on the transformed model
-  
-##   if (model == "within"){
-##     y <- BIGW$y
-##     X <- BIGW$X
-##     m <- lm(y~X - 1)
-##   }
-##   if (model == "between"){
-##     y <- BIGB$y
-##     X <- BIGB$X
-##     m <- lm(y~X - 1)
-##   }
-##   if (model == "random"){
-##     y <- BIGW$y + BIGB$y
-##     X <- BIGW$X + BIGB$X
-##     m <- lm(y~X - 1)
-##   }
-##   return(m)
-## }
-  
